package com.dayou;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelReader;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.read.listener.PageReadListener;
import com.alibaba.excel.read.metadata.ReadSheet;
import com.dayou.utils.EasyExcelUtil;
import com.dayou.utils.ExcelConvertToHtmlUtil;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.junit.jupiter.api.Test;

import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ExcelSimpleTests {

    /**
     * 测试使用EasyExcel填充值到带公式的excel模板
     */
     @Test
    void testEasyExcel(){
        // 模板文件路径
        String templateFilePath = "E:\\test\\input.xlsx";
        // 输出文件路径
        String outputPath = "E:\\test\\output.xlsx";

        // 创建一个Map来存储要填充的数据
        // map中的key是在excel文件设置占位符,比如这里的key是a,那么在excel中的占位就写作 {a}
        Map<String, Object> dataMap = new HashMap<>();
        dataMap.put("a", 13.7);
        dataMap.put("b", 79.78);

        EasyExcelUtil.fillExcelDataByMap(templateFilePath, outputPath, dataMap, true);
    }

    /**
     * 测试使用poi合并Excel文件
     * 大概就是读取模板文件的行信息(包括数据,样式等等),再写入到目标文件中
     */
     @Test
    void testExcelMerge() throws IOException {
        String sourceExcelPath = "E:\\test\\source.xlsx"; // 源Excel文件路径
        String targetExcelPath = "E:\\test\\target.xlsx"; // 目标Excel文件路径
        int targetSheetIndex = 0; // 目标sheet索引
        int targetRowNum = 4; // 目标行索引

         Workbook workbook = EasyExcelUtil.mergeExcel(sourceExcelPath, targetExcelPath, targetSheetIndex, targetRowNum, 0);
         workbook.write(Files.newOutputStream(Paths.get("E:\\test\\merge.xlsx")));
    }

    @Data
    @NoArgsConstructor
    public static class ReadExcel {

        @ExcelProperty("id")
        private Integer id;

        @ExcelProperty("值1")
        private Double value1;

        @ExcelProperty("值2")
        private Double value2;

        @ExcelProperty("和")
        private Double sum;
    }

    /**
     * EasyExcel读取excel表数据内容
     */
    @Test
    void testExcelRead(){
        // 文件路径
        String filePath = "E:\\test\\excel\\read.xlsx";
        // 分页读取
        // PageReadListener默认设置的分页读取是100条,但是可以通过构造函数设置
        EasyExcel.read(filePath, ReadExcel.class, new PageReadListener<ReadExcel>(dataList -> {
            System.out.println(dataList.size());
        }, 1000)).sheet().doRead();
    }

    /**
     * EasyExcel读取excel指定sheet表数据内容
     * 可以是sheet下表也可以是sheet名字
     */
    @Test
    void testExcelReadSheet(){
        // 文件路径
        String filePath = "E:\\test\\excel\\read.xlsx";
        ExcelReader excelReader = EasyExcel.read(filePath).build();
        // 指定sheet
        ReadSheet readSheet = EasyExcel.readSheet("Sheet2").head(ReadExcel.class).registerReadListener(new PageReadListener<ReadExcel>(dataList -> {
            System.out.println(dataList.size());
        }, 1000)).build();

        excelReader.read(readSheet);
    }

    /**
     * EasyExcel读取excel同步返回所有内容
     */
    @Test
    void testExcelReadAll(){
        // 文件路径
        String filePath = "E:\\test\\excel\\read.xlsx";
        // 同步返回读取(酌情使用,数据量大可能会OOM)
        List<ReadExcel> list = EasyExcel.read(filePath).head(ReadExcel.class).sheet().doReadSync();
        System.out.println(list.size());
    }

    /**
     * 测试将excel文件转换成html
     * @throws Exception
     */
    @Test
    void test() throws Exception {
        String sourcePath = "E:\\test\\excel\\房地产Excel.xlsx";
        String htmlLocation =  "E:\\test\\excel\\房地产Excel.html";
        // 将excel转为html并保存为html文件
        System.out.println(ExcelConvertToHtmlUtil.readExcelToHtmlFile(sourcePath, htmlLocation, true, "xlsx", "Sheet1"));
        // 将excel转为html并返回字符串
        System.out.println(ExcelConvertToHtmlUtil.readExcelToHtmlCode(sourcePath, true, "xlsx","Sheet1"));
    }

    /**
     * 测试给excel设置公式
     * @throws IOException
     */
    @Test
    void testSetExcelFormula() throws IOException {
        Workbook workbook = new XSSFWorkbook();
        Sheet sheet = workbook.createSheet("Example Sheet");
        Row row = sheet.createRow(0);
        Cell cellA = row.createCell(0);
        cellA.setCellValue(10);
        Cell cellB = row.createCell(1);
        cellB.setCellValue(20);
        Cell cellSum = row.createCell(2);
        cellSum.setCellFormula("A1+B1"); // 设置公式
        cellSum.setCellType(CellType.FORMULA);

        try (FileOutputStream outputStream = new FileOutputStream("E:\\test\\excel\\Example.xlsx")) {
            workbook.write(outputStream);
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            workbook.close();
        }
    }

}
