package com.dayou;

import com.aspose.words.*;
import com.aspose.words.net.System.Data.DataRow;
import com.aspose.words.net.System.Data.DataTable;
import com.dayou.annotation.WordTableColumn;
import com.dayou.utils.AsposeWordUtil;
import lombok.Builder;
import lombok.Data;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.junit.jupiter.api.Test;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;

/**
 * 测试类引用的常用方法已经封装在AsposeWordUtil工具类
 * 该测试类操作的文档以债权报告为例
 * 参考文档：reference.aspose.com/words/zh/java/com.aspose.words/documentbuilder/
 * -----------------------------------------------------------------------------
 * 1.aspose动态填充是采取的更新域（域替换），目录也是域，动态填充时有目录的话，目录样式会被更新掉，无法设置，所以动态填充时不能有目录
 * 2.aspose设置的页码无法选择从哪一页开始计数，所以页眉页脚与页码需要在模板中提前设置好新增页面的页眉页脚要同上一页页面。无论是拼接上去的word还是插入进去的word都可以顺利继承页眉页脚
 * 3.段落模板设置的页眉页脚，用将段落模板插入到某个位置是无效的，页眉页脚带不过去
 * 4.由于目录的页码与正文内容多数情况采取的不同样式，如果将目录与正文分开拼接，会导致页码样式丢失，最好是目录和一部分正文内容在一起（如像债权一样将目录和声明作为一个段落模板）。
 * 5.该案例过程大概是 填充封面模板内容-->设置主体框架模板的页眉页脚等(框架主要有目录、声明、摘要、正文四部分，这四部分要提前设置好格式)-->分别向框架的四个部分插入替换好值的段落模板-->将框架拼接在封面后-->更新目录页码
 */
public class WordSimpleTests {


    /**
     * 这里的属性名字要和word设置的域名一致
     */
    @Data
    @Builder
    public static class SimpleWordData {
        private String name1;

        private String name2;

        private String productionNo;

        private String date;

        private List<SimpleWordTableData> contractList;
    }

    @Data
    @Builder
    public static class SimpleWordTableData {

        @WordTableColumn(name = "id", fontName = "华文细黑", fontSize = 5.5, titleBold = true, titleFontSize = 8)
        private String id;

        @WordTableColumn(name = "担保人", fontName = "华文细黑", fontSize = 8, titleBold = true)
        private String pledger;

        @WordTableColumn(name = "合同编号", fontName = "华文细黑", fontSize = 8, titleBold = true)
        private String contractNum;

        @WordTableColumn(name = "担保方式", fontName = "华文细黑", fontSize = 8, titleBold = true)
        private String guarantyStyle;

        @WordTableColumn(name = "金额", fontName = "华文细黑", fontSize = 8, titleBold = true)
        private String amount;
    }

    /**
     * 测试向word模板中动态填充数据
     * 此处案例是设置封面
     *
     * @throws IOException
     */
    @Test
    void testWordFilling() throws IOException {
        // 设置信息
        SimpleWordData wordData = SimpleWordData.builder()
                .name1("大连银行股份有限公司成都分行")
                .name2("四川宏淼科技发展有限公司债权")
                .productionNo("川友评报字（2020）第0123-1号")
                .date("二〇二〇年四月三十日")
                .build();

        // 读取模板文件
        byte[] tmplWordByte = Files.readAllBytes(Paths.get("E:\\test\\word\\1_cover.doc"));

        // 获取填充数据后的文件(AsposeWordUtil是工具类,放在common中)
        byte[] resultWordByte = AsposeWordUtil.fillWordDataByDomain(tmplWordByte, wordData);

        // 处理该二进制文件并保存
        File resultFile = new File("E:\\test\\word\\1_cover_output.doc");
        FileOutputStream fos = new FileOutputStream(resultFile);
        fos.write(resultWordByte);
        fos.close();
    }

    /**
     * 测试拼接文档
     * 此处案例是拼接目录与声明的内容
     */
    @Test
    void testAppendWord() {
        try {
            //加载源文档
            Document doc1 = new Document("E:\\test\\word\\1_cover_output.doc"); // 封面
            Document doc2 = new Document("E:\\test\\word\\2_framework_output.doc"); // 目录与声明

            //将第二个文档的内容附加到第一个文档
            doc1.appendDocument(doc2, ImportFormatMode.KEEP_SOURCE_FORMATTING);
            //保存合并的文档
            doc1.save("E:\\test\\word\\6_success_output.doc");
        } catch (Exception e) {
            System.out.println("An error occurred: " + e.getMessage());
            e.printStackTrace();
        }
    }

    /**
     * 测试向word模板中动态填充数据(带集合的情况)
     * 此处案例是设置正文内容
     *
     * @throws IOException
     */
    @Test
    void testWordFillSection() throws IOException {
        // 设置信息
        SimpleWordTableData data1 = SimpleWordTableData.builder().id("1").pledger("张三").contractNum("合同阿巴阿巴阿巴阿巴").guarantyStyle("担保").amount("1000万").build();
        SimpleWordTableData data2 = SimpleWordTableData.builder().id("2").pledger("李四").contractNum("合同呜呼呜呼呜呼芜湖").guarantyStyle("抵押").amount("200028万").build();
        List<SimpleWordTableData> contractList = Arrays.asList(data1, data2);
        SimpleWordData wordData = SimpleWordData.builder()
                .name1("大连银行股份有限公司成都分行")
                .name2("四川宏淼科技发展有限公司债权")
                .productionNo("川友评报字（2020）第0123-1号")
                .date("二〇二〇年四月三十日")
                .contractList(contractList)
                .build();


        // 读取模板文件
        byte[] tmplWordByte = Files.readAllBytes(Paths.get("E:\\test\\word\\5_main.doc"));

        // 获取填充数据后的文件(AsposeWordUtil是工具类,放在common中)
        byte[] resultWordByte = AsposeWordUtil.fillWordDataByDomain(tmplWordByte, wordData);

        // 处理该二进制文件并保存
        File resultFile = new File("E:\\test\\word\\5_main_output.doc");
        FileOutputStream fos = new FileOutputStream(resultFile);
        fos.write(resultWordByte);
        fos.close();
    }

    /**
     * 测试向word指定位置插入另一个word
     * 实测该方法无法插入页眉页脚
     * 此处案例是将正文内容插入到拼接好目录与声明的报告中
     *
     * @throws Exception
     */
    @Test
    void testInsertWord() throws Exception {

        //设置书签，指定文档拼接的位置(书签需要在模板中提前设置好,相当于标识符)
        String bookmark = "insertMain";

        // 主word文件
        Document mainDoc = new Document("E:\\test\\word\\2_framework_output.doc");

        // 需要插入的word文件
        Document subDoc = new Document("E:\\test\\word\\5_main_output.doc");

        mainDoc = AsposeWordUtil.insertDocumentAfterBookMark(mainDoc, subDoc, bookmark, false, true);
        // 保存文件
        mainDoc.save("E:\\test\\word\\2_framework_output.doc");
    }

    /**
     * 更新目录页码
     * 此处案例是更新目录页码
     */
    @Test
    void testUpdateCatalogueLink() throws Exception {
        Document doc = new Document("E:\\test\\word\\6_success_output.doc");
        // 还有一种方式是 doc.updateFields(); 方法,该方法会导致目录样式被更新掉。方便,但是不建议使用
        for (Field field : doc.getRange().getFields()) {
            if (field.getType() == FieldType.FIELD_TOC) {
                FieldToc toc = (FieldToc) field;
                toc.updatePageNumbers(); //定位toc只更新页码
            } else {
                field.update();
            }
        }
        doc.save("E:\\test\\word\\6_success_output.doc");
    }

    /**
     * 测试在指定书签处创建table
     * 这里测试的是从无到有的创建一个table,和值填充到模板中已存在的table不一样
     *
     * @throws Exception
     */
    @Test
    void testCreateTable() throws Exception {
        Document doc = new Document("E:\\test\\word\\createTable.doc");
        // 设置信息
        SimpleWordTableData data1 = SimpleWordTableData.builder().id("1").pledger("张三").contractNum("合同阿巴阿巴阿巴阿巴").guarantyStyle("担保").amount("1000万").build();
        SimpleWordTableData data2 = SimpleWordTableData.builder().id("2").pledger("李四").contractNum("合同呜呼呜呼呜呼芜湖").guarantyStyle("抵押").amount("200028万").build();
        List<SimpleWordTableData> contractList = Arrays.asList(data1, data2);

//        AsposeWordUtil.createTable(doc, SimpleWordTableData.class, contractList, "table");

        DocumentBuilder builder = new DocumentBuilder(doc);
        int tableIndex=0;
        //移动至第二行第一列
        builder.moveToCell(tableIndex,1,0,0);
        builder.getCellFormat().setVerticalMerge(CellMerge.FIRST);//合并的第一个单元格 保留的值是第一个单元格的值
        //移动至第三行第一列
//        builder.moveToCell(tableIndex, 10, 0,0);
//        builder.getCellFormat().setVerticalMerge(CellMerge.PREVIOUS);//被合并的单元格
        for(int i = 2; i < 55; i++ ) {
            builder.moveToCell(0, i, 0, 0);
            builder.getCellFormat().setVerticalMerge(CellMerge.PREVIOUS);
        }
        doc.save("E:\\test\\word\\createTable-test.doc");
    }

    /**
     * 测试将html转成word的文件插入到word中
     * 如果插入后发现table样式收到影响,请检查是否是插入位置设置了段落样式等影响到了table
     *
     * @throws Exception
     */
    @Test
    void testInsertWordTable() throws Exception {
        //设置书签，指定文档拼接的位置(书签需要在模板中提前设置好,相当于标识符)
        String bookmark = "insertTable";

        // 主word文件
        Document mainDoc = new Document("E:\\test\\word\\6_success_output.doc");

        // 需要插入的word文件
        Document subDoc = new Document("E:\\test\\word\\1730712433605.docx");

        mainDoc = AsposeWordUtil.insertDocumentAfterBookMark(mainDoc, subDoc, bookmark, false, true);
        // 保存文件
        mainDoc.save("E:\\test\\word\\createTable.docx");
    }

    /**
     * (该方法实测不是很好用,建议先在模板设置好页眉页脚页码等相关设置)
     * 设置Word页眉
     * 不能先设置页脚，需要先设置页码后才能设置页脚内容
     * 否则提前移动光标到页脚会报错:Cannot insert a node of this type at this location.
     *
     * @throws Exception
     */
//    @Test
    void setHeaderFooter() throws Exception {
        Document mainDoc = new Document("E:\\test\\word\\output.docx");
        DocumentBuilder builder = new DocumentBuilder(mainDoc);
        // 为true表示文档的奇数页和偶数页有不同的页眉和页脚
        builder.getPageSetup().setOddAndEvenPagesHeaderFooter(false);

        /*
         * 设置样式前,先移动光标!
         * 先移动光标!
         * 先移动光标!
         * 先移动光标!
         */

        // 移动光标到页眉
        builder.moveToHeaderFooter(HeaderFooterType.HEADER_PRIMARY);

        // 跳过第一页
        builder.getPageSetup().setDifferentFirstPageHeaderFooter(true);

        // 内容居中
        builder.getParagraphFormat().setAlignment(ParagraphAlignment.CENTER);
        //设置页眉上下边距(用的pt(磅)为单位)
        builder.getPageSetup().setHeaderDistance(45.3536);

        // 添加页眉线
        Border borderHeader = builder.getParagraphFormat().getBorders().getBottom();
        // 设置阴影
        borderHeader.setShadow(true);
        // 设置间距
        borderHeader.setDistanceFromText(2);
        // 设置页眉线风格(此处单线)
        borderHeader.setLineStyle(LineStyle.SINGLE);

        // 字体宋体
        builder.getFont().setName("宋体");
        // 是否加粗
        builder.getFont().setBold(false);
        // 字号(9是小五字号)
        builder.getFont().setSize(9);
        // 设置页头内容
        builder.write("大连银行股份有限公司成都分行拟处置不良资产所涉及的四川宏淼科技发展有限公司");
        builder.writeln();
        builder.write("债权价值分析报告·正文");

        ParagraphCollection paras = mainDoc.getFirstSection().getBody().getParagraphs();
        paras.get(0).remove();

        mainDoc.save("E:\\test\\word\\output.docx");
    }

    /**
     * 设置页脚与页码(该方法实测不是很好用,建议先在模板设置好页眉页脚页码等相关设置)
     *
     * @throws Exception
     */
//    @Test
    void setPageNumber() throws Exception {
        Document doc = new Document("E:\\test\\word\\output.docx");
        DocumentBuilder builder = new DocumentBuilder(doc);

        // 移动光标到页脚
        builder.moveToHeaderFooter(HeaderFooterType.FOOTER_PRIMARY);

        // 跳过第一页(该设置对当前页码无效,跳过首页之后,第二页的页码依旧是2)
        builder.getPageSetup().setDifferentFirstPageHeaderFooter(true);

        builder.getPageSetup().setFooterDistance(28.3465);

        // 添加页脚线
        Border borderFooter = builder.getParagraphFormat().getBorders().getTop();
        // 设置阴影
        borderFooter.setShadow(true);
        // 设置间距
        borderFooter.setDistanceFromText(2);
        // 设置页眉线风格(此处单线)
        borderFooter.setLineStyle(LineStyle.SINGLE);

        // 字体宋体
        builder.getFont().setName("宋体");
        // 是否加粗
        builder.getFont().setBold(false);
        // 字号(9是小五字号)
        builder.getFont().setSize(9);

        // 设置页脚内容
        builder.write("四川大友房地产土地资产评估有限公司\t\t\t");

        doc.getFirstSection().getPageSetup().setSectionStart(SectionStart.CONTINUOUS);
        doc.getFirstSection().getPageSetup().setPageStartingNumber(2);

        doc.save("E:\\test\\word\\output.docx");
    }

    /**
     * 测试删除光标所在位置的段落
     * @throws Exception
     */
    @Test
    void testDeleteParagraph() throws Exception {
        Document doc = new Document("E:\\test\\word\\6_success_output.doc");
        DocumentBuilder builder = new DocumentBuilder(doc);

        // 移动光标到文档的某个特定位置,以文档末尾为例
//        builder.moveToDocumentEnd();
        builder.moveToMergeField("deleteRow");

        // 获取当前光标所在的段落
        Paragraph currentParagraph = builder.getCurrentParagraph();

        currentParagraph.remove();

        doc.save("E:\\test\\word\\delete_row.doc");

    }

    @Test
    void streamCreateContent() throws Exception {
        Document doc = new Document("/Users/wuwei/opt/temp/ct.docx");
        DocumentBuilder builder = new DocumentBuilder(doc);
        builder.moveToBookmark("stream", true, false);
        //builder.moveToDocumentEnd();
        builder.getFont().setName("宋体");
        // 是否加粗
        builder.getFont().setBold(false);
        builder.getFont().setSize(12);
        builder.writeln("这是一个测试段落");
        // 设置信息
        SimpleWordTableData data1 = SimpleWordTableData.builder().id("1").pledger("张三").contractNum("合同阿巴阿巴阿巴阿巴").guarantyStyle("担保").amount("1000万").build();
        SimpleWordTableData data2 = SimpleWordTableData.builder().id("2").pledger("李四").contractNum("合同呜呼呜呼呜呼芜湖").guarantyStyle("抵押").amount("200028万").build();
        List<SimpleWordTableData> contractList = Arrays.asList(data1, data2);
        AsposeWordUtil.createTable(builder, SimpleWordTableData.class, contractList);
        doc.save("/Users/wuwei/opt/temp/ct1.docx");

    }

}
