package com.dayou.utils;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.dayou.dto.Address;

import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * 行政地址工具类
 */
public class AddressUtil {

    private static final Pattern PROVINCE_PATTERN = Pattern.compile("(?<province>.+省|.+自治区|.+市)");
    private static final Pattern CITY_PATTERN = Pattern.compile("(?<city>.+市|.+自治州|.+盟)");
    private static final Pattern DISTRICT_PATTERN = Pattern.compile("(?<district>.+区|.+县|.+旗)");
    private static final Pattern TOWN_PATTERN = Pattern.compile("(?<town>.+街道|.+镇|.+乡)");
    private static final Pattern COMMUNITY_PATTERN = Pattern.compile("(?<community>.+社区|.+村)");
    private static final Pattern ROAD_PATTERN = Pattern.compile("(?<road>.+路.+?段|.+路|.+巷|.+?街.+?段|.+?街)");
    private static final Pattern ROAD_NUMBER_PATTERN = Pattern.compile("(?<roadNumber>\\d+号)");
    private static final Pattern BUILDING_PATTERN = Pattern.compile("(?<building>\\d+楼|\\d+栋|\\d+幢)");
    private static final Pattern UNIT_PATTERN = Pattern.compile("(?<unit>\\d+单元)");
    private static final Pattern FLOOR_PATTERN = Pattern.compile("(?<floor>-?\\d+层|-?\\d+楼)");
    private static final Pattern ROOM_PATTERN = Pattern.compile("(?<room>\\d+室\\d+号|\\d+室|\\d+号)");

    private static final List<String> BIG_CHENGDU  = Arrays.asList("锦江区","武侯区","青羊区","郫都区","金牛区","高新区","温江区","成华区","新都区"
        ,"龙泉驿区","青白江区","双流区","都江堰市","崇州市","邛崃市","简阳市","彭州市","郫县","金堂县","大邑县","浦江县"
        ,"新津县","成都天府新区直管区","成都高新技术产业开发区","成都经济技术开发区");

    private static final List<String> CENTER_CHENGDU  = Arrays.asList("锦江区","武侯区","青羊区","金牛区","成华区");

    private static final List<String> TECH_CHENGDU  = Arrays.asList("高新区","成都高新技术产业开发区");

    /**
     *
     * @param address 地址字符串
     * @return 标准地址对象
     */
    public static Address parseAddress(String address) {
        Address result = new Address();

        // 依次匹配每个级别的地址部分，并从剩余字符串中继续解析
        String remaining = address;
        result.setProvince(extractAndRemove(PROVINCE_PATTERN, remaining));
        remaining = removePrefix(remaining, result.getProvince());

        result.setCity(extractAndRemove(CITY_PATTERN, remaining));
        remaining = removePrefix(remaining, result.getCity());

        result.setDistrict(extractAndRemove(DISTRICT_PATTERN, remaining));
        remaining = removePrefix(remaining, result.getDistrict());

        result.setTown(extractAndRemove(TOWN_PATTERN, remaining));
        remaining = removePrefix(remaining, result.getTown());

        result.setCommunity(extractAndRemove(COMMUNITY_PATTERN, remaining));
        remaining = removePrefix(remaining, result.getCommunity());

        result.setRoad(extractAndRemove(ROAD_PATTERN, remaining));
        remaining = removePrefix(remaining, result.getRoad());

        result.setRoadNumber(extractAndRemove(ROAD_NUMBER_PATTERN, remaining));
        remaining = removePrefix(remaining, result.getRoadNumber());

        result.setBuilding(extractAndRemove(BUILDING_PATTERN, remaining));
        remaining = removePrefix(remaining, result.getBuilding());

        result.setUnit(extractAndRemove(UNIT_PATTERN, remaining));
        remaining = removePrefix(remaining, result.getUnit());

        result.setFloor(extractAndRemove(FLOOR_PATTERN, remaining));
        remaining = removePrefix(remaining, result.getFloor());

        result.setRoom(extractAndRemove(ROOM_PATTERN, remaining));
        remaining = removePrefix(remaining, result.getRoom());

        return result;
    }


    public static List<Address> parseAddress(List<String> addresses) {
        return addresses.stream().map(addr -> {
            return parseAddress(addr);
        }).collect(Collectors.toList());
    }

    private static String extractAndRemove(Pattern pattern, String input) {
        Matcher matcher = pattern.matcher(input);
        if (matcher.find()) {
            return matcher.group().trim();
        }
        return "";
    }

    private static String removePrefix(String input, String prefix) {
        if (prefix == null || input == null) {
            return input;
        }
        return input.replaceFirst(Pattern.quote(prefix), "").trim();
    }

    /**
     * 忽略路以下的单位
     * @param addresses
     * @return
     */
    public static void ignoreToRoad(List<Address> addresses){
        addresses.stream().forEach(x->{
            x.setRoadNumber("");
            x.setBuilding("");
            x.setFloor("");
            x.setUnit("");
            x.setRoom("");
        });
    }

    /**
     * 判断是否属于大成都范围
     * @param address
     * @return
     */
    public static Boolean isInBigChengdu(String address){
        Address addr = parseAddress(address);
        if (addr!=null){
            String city = addr.getCity();
            if (StrUtil.isNotBlank(city) && city.contains("成都")){
                return true;
            }
            String district = addr.getDistrict();

            for (String str : BIG_CHENGDU){
                if (StrUtil.isNotBlank(district) && (str.contains(district) || district.contains(str))){
                    return true;
                }
            }
        }
        return false;

    }

    /**
     * 判断是否属于成都中心城区范围
     * @param address
     * @return
     */
    public static Boolean isCenterChengdu(String address){
        Address addr = parseAddress(address);
        if (addr!=null){
            String city = addr.getCity();
            if (StrUtil.isNotBlank(city) && city.contains("成都")){
                return true;
            }
            String district = addr.getDistrict();

            for (String str : CENTER_CHENGDU){
                if (StrUtil.isNotBlank(district) && (str.contains(district) || district.contains(str))){
                    return true;
                }
            }
        }
        return false;

    }

    /**
     * 判断是否属于成都高新区
     * @param address
     * @return
     */
    public static Boolean isTechChengdu(String address){
        Address addr = parseAddress(address);
        if (addr!=null){
            String city = addr.getCity();
            if (StrUtil.isNotBlank(city) && city.contains("成都")){
                return true;
            }
            String district = addr.getDistrict();

            for (String str : TECH_CHENGDU){
                if (StrUtil.isNotBlank(district) && (str.contains(district) || district.contains(str))){
                    return true;
                }
            }
        }
        return false;

    }


}
