package com.dayou.utils;

import cn.hutool.core.date.DateUtil;
import com.dayou.exception.BusinessException;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import java.net.HttpURLConnection;
import java.net.URL;

@Slf4j
public class FileNetUtil {

    /**
     * 上传文件到服务器本地磁盘(按日期目录存储)
     * @param multiFile
     * @param baseDir
     * @param uploadPath
     * @param uploadFileName
     * @return
     */
    public static Map<String,String> upload(MultipartFile multiFile, String baseDir, String uploadPath, String uploadFileName){
        uploadPath += createDateFolder();
        return uploadToServer(multiFile,baseDir,uploadPath,uploadFileName);
    }

    /**
     * 上传文件到服务器本地磁盘
     *
     * @param multiFile      文件
     * @param baseDir        根路径
     * @param uploadPath     服务器上要存储文件的路径
     * @param uploadFileName 服务器上要存储的文件的名称
     * @return Map<String, String> 文件信息
     */
    public static Map<String, String> uploadToServer(MultipartFile multiFile, String baseDir, String uploadPath, String uploadFileName) {
        //构建文件对象
        File file = new File(baseDir + uploadPath);
        //文件目录不存在则递归创建目录
        if (!file.exists()) {
            boolean mkdirs = file.mkdirs();
            if (!mkdirs) {
                throw new BusinessException("创建文件夹异常");
            }
        }
        try {
            //获取文件输入流
            InputStream inputStream = multiFile.getInputStream();
            //构建文件输出流
            FileOutputStream outputStream = new FileOutputStream(baseDir + uploadPath + uploadFileName);
            // SpringBoot自带工具类上传
            FileCopyUtils.copy(inputStream, outputStream);
            Map<String, String> fileInfo = new HashMap<>();
            fileInfo.put("fileName", uploadFileName);
            fileInfo.put("filePath", uploadPath);
            return fileInfo;
        } catch (IOException e) {
            throw new BusinessException("文件上传异常", e);
        }
    }

    /**
     * 通过url从远端下载文件到服务器本地磁盘
     *
     * @param downloadUrl      要下载的文件的地址
     * @param downloadPath     服务器上存储的文件路径
     * @param downloadFileName 服务器上存储的文件名称
     * @return Map<String, String> 文件信息
     */
    public static Map<String, String> downloadToServer(String downloadUrl, String downloadPath, String downloadFileName) {
        FileOutputStream fos = null;
        BufferedInputStream bis = null;
        try {
            URL url = new URL(downloadUrl);
            HttpURLConnection connection = (HttpURLConnection) url.openConnection();
            connection.connect();
            bis = new BufferedInputStream(connection.getInputStream());
            File file = new File(downloadPath);
            if (!file.exists()) {
                boolean mkdirs = file.mkdirs();
                if (!mkdirs) {
                    throw new BusinessException("创建文件目录失败");
                }
            }
            String filePathName = downloadPath + File.separator + downloadFileName;
            byte[] buf = new byte[1024];
            int size;
            fos = new FileOutputStream(filePathName);
            while ((size = bis.read(buf)) != -1) {
                fos.write(buf, 0, size);
            }
            Map<String, String> fileInfo = new HashMap<>();
            fileInfo.put(downloadFileName, downloadPath);
            return fileInfo;
        } catch (Exception e) {
            throw new BusinessException("下载文件异常", e);
        } finally {
            try {
                if (bis != null) {
                    bis.close();
                }
                if (fos != null) {
                    fos.close();
                }
            } catch (IOException e) {
                throw new BusinessException("关流异常", e);
            }
        }
    }

    /**
     * 创建日期目录
     * @return
     */
    public static String createDateFolder() {
        Date now = new Date();
        String year = DateUtil.format(now, "yyyy");
        String month = DateUtil.format(now, "MM");
        String day = DateUtil.format(new Date(), "dd");
        String time = String.valueOf(System.currentTimeMillis());
        String spliter = "/";
        String index = spliter + year + spliter + month + spliter + day + spliter + time + spliter;
        return index;
    }

    /**
     * 下载本地文件
     * @param request
     * @param response
     * @param file
     * @param fileName
     */
    public static void writeFile2Response(HttpServletRequest request, HttpServletResponse response, File file,
                                     String fileName) {
        try {
            OutputStream out = response.getOutputStream();
            if (request.getHeader("User-Agent").toUpperCase().indexOf("MSIE") > 0) {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } else {
                fileName = new String(fileName.getBytes("UTF-8"), "ISO8859-1");
            }
            response.setHeader("Content-Type", "application/octet-stream");
            response.setHeader("Content-Disposition", "attachment; filename=\"" + fileName + "\"");
            InputStream is = new FileInputStream(file);

            int b = 0;
            byte[] buffer = new byte[1024];
            while (b != -1) {
                b = is.read(buffer);
                if (b != -1) {
                    out.write(buffer, 0, b);
                }
            }
            is.close();

            response.setStatus(HttpServletResponse.SC_OK);
            response.flushBuffer();
            out.close();
        } catch (Exception e) {
            log.error("下载" + fileName + "错误", e);
            throw new RuntimeException(e);
        }
    }

}
