package com.dayou.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.exception.ExcelDataConvertException;
import com.alibaba.excel.read.listener.PageReadListener;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dayou.common.BaseEntity;
import com.dayou.common.CalculateFormula;
import com.dayou.common.DropDownBoxData;
import com.dayou.config.FileNetConfig;
import com.dayou.dto.calculate.equipment.EqptCalculateBaseInfoDTO;
import com.dayou.entity.*;
import com.dayou.exception.BusinessException;
import com.dayou.mapper.AssetsCalculateMapper;
import com.dayou.mapper.TmplAssetCalculateMapper;
import com.dayou.mapper.TmplAssetCalculateSectionMapper;
import com.dayou.service.AssetsCalculateEqptDataService;
import com.dayou.service.AssetsCalculateService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.service.DocumentProductionService;
import com.dayou.utils.EasyExcelUtil;
import com.dayou.vo.*;
import com.dayou.vo.calculate.AssetsCalculateProgressVO;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.MessageFormat;
import java.time.LocalDate;
import java.time.format.DateTimeParseException;
import java.util.List;

import static com.dayou.common.CalculateFormula.*;
import static com.dayou.enums.DocumentType.EQPT_CALCULATE;
import static com.dayou.enums.EqptCalculateTmplCode.*;

/**
 * <p>
 * 资产项目测算信息表 服务实现类
 * </p>
 *
 * @author mybatis-plus-generator
 * @since 2024-11-13
 */
@Service
public class AssetsCalculateServiceImpl extends ServiceImpl<AssetsCalculateMapper, AssetsCalculate> implements AssetsCalculateService {

    @Autowired
    private AssetsCalculateMapper assetsCalculateMapper;

    @Autowired
    private AssetsCalculateEqptDataService dataService;

    @Autowired
    private TmplAssetCalculateMapper tmplAssetCalculateMapper;

    @Autowired
    private TmplAssetCalculateSectionMapper calculateSectionMapper;

    @Autowired
    private DocumentProductionService documentProductionService;

    @Autowired
    private FileNetConfig fileNetConfig;

    /**
     * 根据项目id查询所有测算表信息
     *
     * @param projectId 项目id
     * @return List<AssetsCalculateVO>
     */
    @Override
    public List<AssetsCalculateVO> listAllByProjectId(Long projectId) {
        return assetsCalculateMapper.listAllByProjectId(projectId);
    }

    @Override
    public AssetsCalculate detail(Long id) {
        return this.getById(id);
    }

    /**
     * 新增或修改测算表信息
     *
     * @param assetsCalculate 测算表信息
     * @return Boolean
     */
    @Override
    public Boolean add(AssetsCalculate assetsCalculate) {
        assetsCalculate.setCreateUserId(StpUtil.getLoginIdAsLong());
        if (ObjectUtil.isNull(assetsCalculate.getId())) {
            return this.save(assetsCalculate);
        } else {
            return this.updateById(assetsCalculate);
        }
    }

    @Override
    public Boolean update(AssetsCalculate assetsCalculate) {
        return this.updateById(assetsCalculate);
    }

    @Override
    public Boolean delete(Long id) {
        //逻辑删除
        return this.removeById(id);
    }

    /**
     * 根据项目id获取未完成测算表的进度信息
     *
     * @param projectId 项目id
     * @return AssetsCalculateStepBaseVO
     */
    @Override
    public AssetsCalculateProgressVO getUnFinishedCalculateProgress(Long projectId) {
        return assetsCalculateMapper.getCalculateProgress(projectId);
    }

    /**
     * 导入保存机器设备基础信息
     *
     * @param multipartFile Excel文件
     * @param calculateId   测算表id
     * @return Boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void importEqptExcelData(MultipartFile multipartFile, Long calculateId) throws IOException {
        // 删除已上传的数据
        dataService.deleteDataByCalculateId(calculateId);
        // 读取excel内容并批量保存到数据库
        InputStream inputStream = multipartFile.getInputStream();
        try {
            EasyExcel.read(inputStream, AssetsCalculateEqptData.class, new PageReadListener<AssetsCalculateEqptData>(dataList -> {
                // 设置数据的测算表id
                dataList.forEach(data -> data.setAssetsCalculateId(calculateId));
                // 批量保存
                dataService.saveBatch(dataList);
            }, 1000)).sheet().doRead();
        } catch (ExcelDataConvertException dataConvertException) {
            // 处理Excel数据转换异常
            if (dataConvertException.getCause() instanceof DateTimeParseException) {
                DateTimeParseException dateException = (DateTimeParseException) dataConvertException.getCause();
                // 处理日期时间解析异常
                throw new BusinessException("导入的Excel表中有不规范的日期格式,请检查后重试!", dateException);
            } else {
                // 处理其他类型的转换异常
                throw new BusinessException("导入的Excel表中有不规范的数据,请检查后重试!", dataConvertException);
            }
        }
        // 更新测算表进度
        assetsCalculateMapper.updateCalculateProgress(calculateId, "GENERATE");
    }

    /**
     * 更新测算表基础测算信息
     *
     * @param calculateId id
     * @param baseInfo    基础信息json字符串
     * @return Boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateCalculateBaseInfo(Long calculateId, String baseInfo) {
        // 如果baseInfo为null说明是第一次进行填写测算基本信息的操作,则更新测算表进度
        if (ObjectUtil.isNull(getCalculateBaseInfo(calculateId))) {
            assetsCalculateMapper.updateCalculateProgress(calculateId, "IMPORT");
        }
        return assetsCalculateMapper.updateCalculateBaseInfo(calculateId, baseInfo);
    }

    /**
     * 生成机器设备测算表excel文件
     *
     * @param calculateId 测算表id
     * @return Boolean
     */
    @Override
    public Boolean generateEquipmentCalculate(Long calculateId) throws IOException {

        // 第一步:写入数据到 机器设备-主模板-设备信息-段落模板
        // 获取模板
        TmplAssetCalculateSection mainDetail = calculateSectionMapper.getByTmplCode(MAIN_DETAIL.getCode());
        // 生成后文件的位置
        String mainDetailPath = fileNetConfig.getBaseDir() + fileNetConfig.getAssetOutputCalculatePath() + System.currentTimeMillis() + "_MAIN_DETAIL.xlsx";
        // 模板文件的位置
        String mainDetailTmplPath = fileNetConfig.getBaseDir() + mainDetail.getSectionFileUrl() + mainDetail.getSectionFileName();
        // 写入
        List<AssetsCalculateEqptData> allEqptData = dataService.getAllByAssetsCalculateId(calculateId, null);
        EasyExcel.write(mainDetailPath)
                .withTemplate(mainDetailTmplPath)
                .sheet()
                .doFill(allEqptData);

        // 第二步:写入数据到 机器设备-主模板
        // 获取模板
        TmplAssetCalculate main = tmplAssetCalculateMapper.getCalculateTmplByCode(MAIN.getCode());
        // 生成后文件的位置
        String mainPath = fileNetConfig.getBaseDir() + fileNetConfig.getAssetOutputCalculatePath() + System.currentTimeMillis() + "_MAIN.xlsx";
        // 模板文件的位置
        String mainTmplPath = fileNetConfig.getBaseDir() + main.getFileUrl() + main.getFileName();
        // 获取基础测算信息
        ObjectMapper objectMapper = new ObjectMapper();
        EqptCalculateBaseInfoDTO baseInfoDTO = objectMapper.readValue(getCalculateBaseInfo(calculateId), EqptCalculateBaseInfoDTO.class);
        // 利息、前期费用率、建设单位管理费率采用百分比,需要除100
        baseInfoDTO.setInterest(baseInfoDTO.getInterest() / 100);
        baseInfoDTO.setPreConstructionCostRatio(baseInfoDTO.getPreConstructionCostRatio() / 100);
        baseInfoDTO.setManagementExpenseRatio(baseInfoDTO.getManagementExpenseRatio() / 100);
        // 获取评估基准日
        baseInfoDTO.setValuationBasisDate(getValuationBasisDate(calculateId));
        // 写入
        EasyExcel.write(mainPath)
                .withTemplate(mainTmplPath)
                .sheet()
                .doFill(baseInfoDTO);

        // 第三步:将 机器设备-主模板-设备信息-段落模板 拼接到 写入数据到 机器设备-主模板
        // 合并后文件的位置
        String docUrl = fileNetConfig.getAssetOutputCalculatePath() + System.currentTimeMillis() + "_机器设备测算表.xlsx";
        String mainMergePath = fileNetConfig.getBaseDir() + docUrl;
        Workbook workbook = EasyExcelUtil.mergeExcel(mainDetailPath, mainPath, 0, 4, 0, 0);

        // 第四步:主模板设置公式
        // 遍历所有行,跳过前四行
        Sheet sheet = workbook.getSheetAt(0);

        // 修复第一行数据公式引用的数据为NULL时公式报错 #VALUE
        if (ObjectUtil.isNull(allEqptData.get(0).getFreightAndMiscellaneousFeeRate())) {
            sheet.getRow(4).getCell(36).setBlank();
        }
        if (ObjectUtil.isNull(allEqptData.get(0).getBaseRate())) {
            sheet.getRow(4).getCell(38).setBlank();
        }
        if (ObjectUtil.isNull(allEqptData.get(0).getInstallationRate())) {
            sheet.getRow(4).getCell(40).setBlank();
        }

        for (int i = 4; i <= sheet.getLastRowNum(); i++) {
            Row row = sheet.getRow(i);
            // 循环设置公式
            for (CalculateFormula.EquipmentMainFormula formula : EQUIPMENT_MAIN_FORMULA) {
                Cell cell = row.getCell(formula.getColumn() - 1);
                // 如果该单元格不存在,则创建一个
                if (cell == null) {
                    cell = row.createCell(formula.getColumn() - 1);
                }
                // 设置公式
                cell.setCellFormula(MessageFormat.format(formula.getFormula(), (row.getRowNum() + 1)));
            }
        }

        // 第五步:设置进口设备的测算表
        List<AssetsCalculateEqptImpDataVO> impEqptData = dataService.getImpDataList(calculateId);
        if (ObjectUtil.isNotNull(impEqptData) && ObjectUtil.isNotEmpty(impEqptData)) {
            // 获取模板
            TmplAssetCalculateSection imported = calculateSectionMapper.getByTmplCode(IMPORTED.getCode());
            // 生成后文件的位置
            String importedPath = fileNetConfig.getBaseDir() + fileNetConfig.getAssetOutputCalculatePath() + System.currentTimeMillis() + "_IMPORTED.xlsx";
            // 模板文件的位置
            String importedTmplPath = fileNetConfig.getBaseDir() + imported.getSectionFileUrl() + imported.getSectionFileName();
            // 写入
            EasyExcel.write(importedPath)
                    .withTemplate(importedTmplPath)
                    .sheet()
                    .doFill(impEqptData);
            // 将进口设备excel合并到主excel
            workbook.createSheet("进口设备");
            EasyExcelUtil.mergeExcel(importedPath, workbook, workbook.getSheetIndex("进口设备"), 0, 0, 0);
            // 设置公式
            Sheet importedSheet = workbook.getSheet("进口设备");

            // 修复第一行数据公式引用的数据为NULL时公式报错 #VALUE
            if (ObjectUtil.isNull(impEqptData.get(0).getBankChargesRate())) {
                importedSheet.getRow(2).getCell(27).setBlank();
            }
            if (ObjectUtil.isNull(impEqptData.get(0).getForeignTransactionRate())) {
                importedSheet.getRow(2).getCell(29).setBlank();
            }

            for (int i = 2; i <= importedSheet.getLastRowNum(); i++) { // 跳过前两行
                Row row = importedSheet.getRow(i);
                // 循环设置公式
                for (CalculateFormula.EquipmentImportedFormula formula : EQUIPMENT_IMPORTED_FORMULA) {
                    Cell cell = row.getCell(formula.getColumn() - 1);
                    // 如果该单元格不存在,则创建一个
                    if (cell == null) {
                        cell = row.createCell(formula.getColumn() - 1);
                    }
                    // 设置公式
                    cell.setCellFormula(MessageFormat.format(formula.getFormula(), (row.getRowNum() + 1)));
                }
            }

            // 将主表sheet中进口设备数据的单价改为引用对应的进口设备表的设备原价
            for (Row row : sheet) {
                Cell cell = row.getCell(29); // AB列(设备类型)29（从0开始计数）
                if (cell != null && cell.getStringCellValue().equals("进口设备")) {
                    Cell priceCell = row.getCell(34); // 单价单元格
                    int importedRowNum;
                    // 循环进口设备sheet
                    for (int i = 2; i <= importedSheet.getLastRowNum(); i++) { // 跳过前两行
                        Cell importedCell = importedSheet.getRow(i).getCell(0); // 获取进口设备的序号
                        // 判断主表的序号和进口设备的序号是否相等
                        if (importedCell != null && row.getCell(0).getNumericCellValue() == importedCell.getNumericCellValue()) {
                            importedRowNum = importedCell.getRowIndex() + 1; // +1是因为代码中下标从0开始,但是excel是从1开始
                            priceCell.setCellFormula(MessageFormat.format("进口设备!AG{0}", importedRowNum));
                            break;
                        }
                    }
                }
            }

            // 删除进口设备临时文件
            Files.delete(Paths.get(importedPath));
        }

        // 第六步:设置非标设备的测算表
        List<AssetsCalculateEqptNonsDataVO> nonsEqptData = dataService.getNonSDataList(calculateId);
        if (ObjectUtil.isNotNull(nonsEqptData) && ObjectUtil.isNotEmpty(nonsEqptData)) {
            // 获取模板
            TmplAssetCalculateSection nonstandard = calculateSectionMapper.getByTmplCode(NONSTANDARD.getCode());
            // 生成后文件的位置
            String nonstandardPath = fileNetConfig.getBaseDir() + fileNetConfig.getAssetOutputCalculatePath() + System.currentTimeMillis() + "_NONSTANDARD.xlsx";
            // 模板文件的位置
            String nonstandardTmplPath = fileNetConfig.getBaseDir() + nonstandard.getSectionFileUrl() + nonstandard.getSectionFileName();
            // 写入
            EasyExcel.write(nonstandardPath)
                    .withTemplate(nonstandardTmplPath)
                    .sheet()
                    .doFill(nonsEqptData);
            // 将进口设备excel合并到主excel
            workbook.createSheet("非标设备");
            EasyExcelUtil.mergeExcel(nonstandardPath, workbook, workbook.getSheetIndex("非标设备"), 0, 0);
            // 设置公式
            Sheet nonstandardSheet = workbook.getSheet("非标设备");
            for (int i = 2; i <= nonstandardSheet.getLastRowNum(); i++) { // 跳过前两行
                Row row = nonstandardSheet.getRow(i);
                // 循环设置公式
                for (CalculateFormula.EquipmentNonStandardFormula formula : EQUIPMENT_NONSTANDARD_FORMULA) {
                    Cell cell = row.getCell(formula.getColumn() - 1);
                    // 如果该单元格不存在,则创建一个
                    if (cell == null) {
                        cell = row.createCell(formula.getColumn() - 1);
                    }
                    // 设置公式
                    cell.setCellFormula(MessageFormat.format(formula.getFormula(), (row.getRowNum() + 1)));
                }
            }

            // 将主表sheet中非标设备数据的单价改为引用对应的非标设备表的设备原价
            for (Row row : sheet) {
                Cell cell = row.getCell(29); // AB列(设备类型)的索引是28（从0开始计数）
                if (cell != null && cell.getStringCellValue().equals("非标设备")) {
                    Cell priceCell = row.getCell(34); // 单价单元格
                    int importedRowNum;
                    // 循环进口设备sheet
                    for (int i = 2; i <= nonstandardSheet.getLastRowNum(); i++) { // 跳过前两行
                        Cell importedCell = nonstandardSheet.getRow(i).getCell(0); // 获取进口设备的序号
                        // 判断主表的序号和进口设备的序号是否相等
                        if (importedCell != null && row.getCell(0).getNumericCellValue() == importedCell.getNumericCellValue()) {
                            importedRowNum = importedCell.getRowIndex() + 1; // +1是因为代码中下标从0开始,但是excel是从1开始
                            priceCell.setCellFormula(MessageFormat.format("非标设备!AC{0}", importedRowNum));
                            break;
                        }
                    }
                }
            }
            // 删除非标设备临时文件
            Files.delete(Paths.get(nonstandardPath));
        }

        // 强制计算公式
        workbook.getCreationHelper().createFormulaEvaluator().evaluateAll();


        // 设置表尾合计(这一步一定要最后做,如果在设置其他公式之前设置表尾合计,可能会导致表尾合计的公式被覆盖)
        TmplAssetCalculateSection mainBottomTotal = calculateSectionMapper.getByTmplCode(MAIN_BOTTOM_TOTAL.getCode());
        EasyExcelUtil.mergeExcel(fileNetConfig.getBaseDir() + mainBottomTotal.getSectionFileUrl() + mainBottomTotal.getSectionFileName(), workbook, 0, workbook.getSheetAt(0).getLastRowNum() + 1, 0, 0);
        workbook.getSheetAt(0).addMergedRegion(new CellRangeAddress(workbook.getSheetAt(0).getLastRowNum(), workbook.getSheetAt(0).getLastRowNum(), 0, 3));
        OutputStream outputStream = Files.newOutputStream(Paths.get(mainMergePath));
        workbook.write(outputStream);
        // 关闭流
        workbook.close();
        outputStream.close();

        // 删除测算表临时文件
        Files.delete(Paths.get(mainDetailPath));
        Files.delete(Paths.get(mainPath));

        // 更新测算表进度
        assetsCalculateMapper.updateCalculateProgress(calculateId, "FINISHED");

        // 保存文档信息到数据库
        AssetsCalculateVO assetsCalculateVO = assetsCalculateMapper.getCalculateInfoForDoc(calculateId);
        // 设置文档信息
        DocumentProduction documentProduction = documentProductionService.getDocProdByBusinessId(assetsCalculateVO.getProjectId(), calculateId);

        if (ObjectUtil.isNotNull(documentProduction)) { // 判断文档信息是否为空,不为空说明以前生成过文档,则更新文档版本号
            documentProduction.setDocVersion(documentProduction.getDocVersion() + 1);
        } else { // 为空重新new对象
            documentProduction = new DocumentProduction();
        }
        // 设置文档信息
        documentProduction.setBusinessType("ASSETS")
                .setBusinessId(assetsCalculateVO.getProjectId())
                .setDocType(EQPT_CALCULATE.getName())
                .setDocName(assetsCalculateVO.getCalculateName() + ".xlsx")
                .setConsignor(assetsCalculateVO.getPrincipal())
                .setDocUrl(docUrl)
                .setCreateUserId(StpUtil.getLoginIdAsLong())
                .setBusinessCate(assetsCalculateVO.getProjectTypeName())
                .setDocNo("")
                .setBusinessSubId(calculateId);

        return documentProductionService.saveDocumentProduction(documentProduction);
    }

    /**
     * 根据测算表id获取基础测算信息
     *
     * @param calculateId 测算表id
     * @return String
     */
    @Override
    public String getCalculateBaseInfo(Long calculateId) {
        return assetsCalculateMapper.getCalculateBaseInfo(calculateId);
    }

    /**
     * 根据测算表id获取评估基准日
     *
     * @param calculateId 测算表id
     * @return String
     */
    @Override
    public LocalDate getValuationBasisDate(Long calculateId) {
        return assetsCalculateMapper.selectOne(new LambdaQueryWrapper<AssetsCalculate>()
                .select(AssetsCalculate::getValuationBasisDate)
                .eq(BaseEntity::getId, calculateId)
                .eq(BaseEntity::getDeleteStatus, 0)).getValuationBasisDate();
    }

    /**
     * 根据项目id获取测算信息下拉框数据
     *
     * @param projectId 项目id
     * @return List<DropDownBoxData>
     */
    @Override
    public List<DropDownBoxData> getDropDownBoxData(Long projectId) {
        return assetsCalculateMapper.getDropDownBoxData(projectId);
    }

    /**
     * 根据测算表id获取生成机器设备融资项目报告引用的测算表信息
     *
     * @param calculateId 测算表id
     * @return EqptReportReferencesCalculateVO
     */
    @Override
    public EqptReportReferencesCalculateVO getEqptReferencesInfoById(Long calculateId) {
        EqptReportReferencesCalculateVO referencesVO = assetsCalculateMapper.getEqptReferencesInfoById(calculateId);
        // 判断最大年份是否等于最小年份
        if (referencesVO.getMinPurchaseYear().equals(referencesVO.getMaxPurchaseYear())) {
            referencesVO.setPurchasePeriod(referencesVO.getMinPurchaseYear() + "年");
        } else {
            referencesVO.setPurchasePeriod(referencesVO.getMinPurchaseYear() + "年至" + referencesVO.getMaxPurchaseYear() + "年");
        }
        return referencesVO;

    }

    /**
     * 根据测算表id获取生成债权分析报告引用的测算表信息
     * @param calculateId 测算表id
     * @return EqptReportReferencesCalculateVO
     */
    @Override
    public CRReportReferencesCalculateVO getCRReferencesInfoById(Long calculateId) {
        return assetsCalculateMapper.getCRReferencesInfoById(calculateId);
    }

    /**
     * 获取新增DocumentProduction所需的测算信息
     * @param calculateId 测算表id
     * @return AssetsCalculateVO
     */
    @Override
    public AssetsCalculateVO getCalculateInfoForDoc(Long calculateId) {
        return assetsCalculateMapper.getCalculateInfoForDoc(calculateId);
    }
}
